/* arch/arm/mach-lh7a40x/include/mach/timex.h
 *
 *  Copyright (C) 2004 Coastal Environmental Systems
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  version 2 as published by the Free Software Foundation.
 *
 */

#include <mach/constants.h>

#define CLOCK_TICK_RATE		(PLL_CLOCK/6/16)

/*
#define CLOCK_TICK_RATE		3686400
*/
